/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
*/

var argscheck = require('cordova/argscheck'),
    channel = require('cordova/channel'),
    utils = require('cordova/utils'),
    exec = require('cordova/exec'),
    cordova = require('cordova');

channel.createSticky('onCordovaInfoReady');
// Tell cordova channel to wait on the CordovaInfoReady event
channel.waitForInitialization('onCordovaInfoReady');

/**
 * This represents the mobile device, and provides properties for inspecting the model, version, UUID of the
 * phone, etc.
 * @constructor
 */
function Device() {
    this.available = false;
    this.platform = null;
    this.version = null;
    this.uuid = null;
    this.cordova = null;
    this.model = null;
    this.manufacturer = null;
    this.isVirtual = null;
    this.serial = null;

    var me = this;

    channel.onCordovaReady.subscribe(function() {
        me.getInfo(function(info) {
            //ignoring info.cordova returning from native, we should use value from cordova.version defined in cordova.js
            //TODO: CB-5105 native implementations should not return info.cordova
            var buildLabel = cordova.version;
            me.available = true;
            me.platform = info.platform;
            me.version = info.version;
            me.uuid = info.uuid;
            me.cordova = buildLabel;
            me.model = info.model;
            me.isVirtual = info.isVirtual;
            me.manufacturer = info.manufacturer || 'unknown';
            me.serial = info.serial || 'unknown';
            channel.onCordovaInfoReady.fire();
        },function(e) {
            me.available = false;
            utils.alert("[ERROR] Error initializing Cordova: " + e);
        });
    });
}

/**
 * Get device info
 *
 * @param {Function} successCallback The function to call when the heading data is available
 * @param {Function} errorCallback The function to call when there is an error getting the heading data. (OPTIONAL)
 */
Device.prototype.getInfo = function(successCallback, errorCallback) {
    argscheck.checkArgs('fF', 'Device.getInfo', arguments);
    exec(successCallback, errorCallback, "Device", "getDeviceInfo", []);
};
/*
 * device.getDeviceInfo 获取设备信息
 * return:
 * hardware: 硬件信息（品牌型号） （例如iPad 2，Nexus 5, Sumsuang 9100）
 * deviceType: 硬件类型 （Desktop/Notebook/Tablet/Handheld）
 * deviceId: 设备ID （PC可取MAC地址，iOS取软件唯一号），可采用与设备服务器id相同的方式。
 * platform：平台，例如Windows, Windows NT, iOS, Android
 * platformVersion：平台版本
 */
Device.prototype.getDeviceInfo = function () {
    var me = this;
    var info =
    {
        Hardware: me.model,
        //DeviceType: me.deviceType,
        DeviceId: me.uuid,
        OS: 'Android',
        Platform: me.platform,
        PlatformVersion: me.version,
        Cordova: me.cordova
    };
    return info;
};

/**
 * Get device info
 *
 * @param {Function} successCallback The function to call when the heading data is available
 * @param {Function} errorCallback The function to call when there is an error getting the heading data. (OPTIONAL)
 */
Device.prototype.getInfo = function (successCallback, errorCallback) {
    argscheck.checkArgs('fF', 'Device.getInfo', arguments);
    exec(successCallback, errorCallback, "Device", "getDeviceInfo", []);
};

/**
 * takePicture
 *
 * @param {Function} successCallback The function to call when the heading data is available
 */
Device.prototype.takePicture = function (src, des, cb) {
    var opt = {};

    if (src == 'lib')
        opt.sourceType = Camera.PictureSourceType.PHOTOLIBRARY;

    if (des == 'base64')
        opt.destinationType = Camera.DestinationType.DATA_URL;


    navigator.camera.getPicture(function (data) {
        cb(undefined, des, data);
    }, function (error) {
        cb({msg: error});
    }, opt);

};

/**
 * takePicture
 *
 * @param {Function} successCallback The function to call when the heading data is available
 */
Device.prototype.isOnline = function () {
    return navigator.onLine;
};

Device.prototype.setAppInstalled = function () {
    exec(null, null, "Device", "setAppInstalled", []);
};

Device.prototype.fullScreen = function (enable) {
    exec(null, null, "Device", enable ? "enableFullScreen" : "disableFullScreen", []);
};

Device.prototype.isOnline = function () {
    return navigator.onLine;
};

Device.prototype.openSystemBrowser = function (url) {
    exec(null, null, "Device", "openSystemBrowser", [url]);
};

Device.prototype.openSystemBrowser = function (url) {
    exec(null, null, "Device", "exit", [url]);
};
module.exports = new Device();
